////////////////////////////////////////////////////////////////////////////
//
// draglist.cpp  This is the main source module for the program
//               DRAGLIST.EXE.  This sample implements drag and drop
//               list boxes using the primitive (3.x) Windows API
//
// Written By :  George Cross, Borland C++ Developer Support
// Dated      :  June 16, 1966
// Compiler   :  Borland C++ 5.0 but is valid for version 4.x
//
/////////////////////////////////////////////////////////////////////////

#define STRICT                    //Enable variable type checking
#include <windows.h>              //Include header for the Windows API
#include "draglist.rh"


////////////////////////////////////////////////////////////////////////////
//
// Global variable definitions.
//
////////////////////////////////////////////////////////////////////////////

HINSTANCE hInst;              //Handle to this instance data segment
HWND ghMainDlg,               //Handle to main window
     ghDragList,              //Handle to listbox to drag from..
     ghDropList;              //Handle to listbox to drop to..
HCURSOR hOldCursor;           //Handle to the default cursor
BOOL CaptureOn;               //Flag indicating mouse capture
char glpszCharString[30];     //Character string buffer
                              //

////////////////////////////////////////////////////////////////////////////
//
// Global function definitions.
//
////////////////////////////////////////////////////////////////////////////

//
//Pointer to the default window procedure for a Listbox, as predefined
//by Windows.
//
LRESULT (FAR PASCAL __export *DefListBoxProc)(HWND, UINT, WPARAM, LPARAM);


////////////////////////////////////////////////////////////////////////////
//
// FUNCTION:    DragListProc ( )
//
// PURPOSE:     Window procedure for the DragList listbox window.
//
// PARAMETERS:  Standard window callback procedure variables,
//              Name       Type     Description
//              ----       ----     -----------
//              hWnd       HWND     handle to the window of this procedure
//              message    UINT     message being sent to this window
//              wParam     WPARAM   first parameter for the message
//              lParam     LPARAM   second parameter for the message.
//
// RETURNS:     A long value dependent on the message sent.
//
// REMARKS:     This is the primary message response function for the
//              ListBox from which items will be dragged to other controls.
//              This function in turn calls the original default window
//              procedure for a Listbox as defined by Windows
//
////////////////////////////////////////////////////////////////////////////
LRESULT FAR PASCAL __export DragListProc (HWND   hWnd,
                                          UINT   message,
                                          WPARAM wParam,
                                          LPARAM lParam )
{
 HCURSOR hCursor;         //Handle to the new cursor
 DWORD dwIndex;           //Index of currently selected item in the listbox

 switch (message)                       //Main switch statement for messages
	{
    case WM_MOUSEMOVE:                  //When the mouse is moved,
		 if (CaptureOn == TRUE)           //If it is being captured, then
       {
          hCursor = LoadCursor (NULL,  //Load a custom cursor
                                MAKEINTRESOURCE(IDC_SIZENWSE));
          hOldCursor = SetCursor (hCursor); //Show it.
		 }                                //End  if (CaptureOn == TRUE)

       break;                           //Exit switch statement

    case WM_LBUTTONDOWN:                //When the left button is pressed
                                        //Call the original default function,
       (*DefListBoxProc)(hWnd, message, wParam, lParam);
                                        //Get the currently selected item,
       dwIndex = SendMessage (hWnd, LB_GETCURSEL, 0, 0L);
		 if (dwIndex == LB_ERR)           //If no item in the list is selected,
       {                                //Then find how many items list has
          dwIndex = SendMessage (hWnd, LB_GETCOUNT, 0, 0L);
          if (dwIndex != LB_ERR)        //If the list is NOT empty, then
          {                             //Select the first item in the list
             SendMessage (hWnd, LB_SETCURSEL, 0, 0L);
             dwIndex = 0;               //Reflect this change using the
          } //End if                    //index variable
          else                          //Otherwise, if the list is empty,
          {                             //Do nothing and exit the switch
           break;                       //statement
			 } //End else
       } //End if (dwIndex != LB_ERR)

       SendMessage (hWnd,               //Retrieve the text of the currently
                    LB_GETTEXT,         //selected item in the Listbox
                    dwIndex,            //Index of selected item
                    (LONG) glpszCharString);  //Buffer to fill with text

       SetCapture (hWnd);       //Capture ensuing mouse events to this window
       CaptureOn = TRUE;        //Raise flag indicating mouse is captured
       break;                   //Exit switch statement

    case WM_LBUTTONUP:                  //When the left button is released
                                        //Call the original default function
       (*DefListBoxProc)(hWnd, message, wParam, lParam);
       if (CaptureOn == TRUE)           //If the mouse is being captured,
       {
          ReleaseCapture ();            //Stop capturing the mouse
          CaptureOn = FALSE;            //Lower the flag indicating so
          SetCursor (hOldCursor);
													 //Get the point where the mouse was
          POINT p = {LOWORD(lParam), HIWORD(lParam)};  //released
                                        //Convert that point to dimensions
                                        //of the main window
          MapWindowPoints (hWnd, ghMainDlg, &p, 1);
                                        //If that point is in the list box
                                        //into which we want items dropped,
          if (ChildWindowFromPoint (ghMainDlg, p) == ghDropList)
          {
              SendMessage (ghDropList,        //Insert the item in the list
                           LB_INSERTSTRING,
                           -1,                      //Last position in list
                           (LONG) glpszCharString); //Buffer containing text
          } //End if (ChildWindowFromPoint . . .)
       } //End if (CaptureOn == TRUE)
       break;                           //Exit switch statement

    default:                            //Passes it on if unproccessed
       return (*DefListBoxProc)(hWnd, message, wParam, lParam);
	} //End switch
	return FALSE;                          //Exit OK
}                                         //End DragListProc


////////////////////////////////////////////////////////////////////////////
//
// FUNCTION:    MainDlgProc ( )
//
// PURPOSE:     Window procedure for the main window of the application
//
// PARAMETERS:  Standard window callback procedure variables,
//              Name       Type     Description
//              ----       ----     -----------
//              hWnd       HWND     handle to the window of this procedure
//              message    UINT     message being sent to this window
//              wParam     WPARAM   first parameter for the message
//              lParam     LPARAM   second parameter for the message.
//
// RETURNS:     FALSE if DefDlgProc is to do further processing.
//              TRUE if the message has been completely handled within
//              this MainDlgProc function.
//
// REMARKS:     This is the primary message response function for the
//              main window of our application. Since our main window
//              is an instance of the predefined Windows "dialog" class,
//              we must return FALSE to indicate the DefDlgProc is to
//              process the message further.
//
////////////////////////////////////////////////////////////////////////////
LRESULT FAR PASCAL __export MainDlgProc(HWND   hWnd,
                                        UINT   message,
                           													WPARAM wParam,
                                        LPARAM lParam)
{
 switch (message)
	{
    case WM_CLOSE:                      //When the window is closed
      DestroyWindow(hWnd);              //destroy it.
      break;
	 case WM_QUIT:                       //When quitting the application
		if (hWnd) DestroyWindow (hWnd);   //Ensure the main window is destroyed
      break;                            //first.
	 case WM_DESTROY:                    //When the main window is destroyed,
		PostQuitMessage(0);               //quit the application
		break;
	 default:                            //Pass the message to DefDlgProc
		return (FALSE);                   //if unproccessed
	} //End switch
	return (TRUE);                       //Indicate the message needs no
} //End MainDlgProc                     //further processing


////////////////////////////////////////////////////////////////////////////
//
// FUNCTION:    InitInstance ( )
//
// PURPOSE:     Window procedure for the main window of the application
//
// PARAMETERS:  Standard window callback procedure variables,
//              Name       Type     Description
//              ----       ----     -----------
//              hWnd       HWND     handle to the window of this procedure
//              message    UINT     message being sent to this window
//              wParam     WPARAM   first parameter for the message
//              lParam     LPARAM   second parameter for the message.
//
// RETURNS:     FALSE if DefDlgProc is to do further processing.
//              TRUE if the message has been completely handled within
//              this MainDlgProc function.
//
// REMARKS:     This is the primary message response function for the
//              main window of our application. Since our main window
//              is an instance of the predefined Windows "dialog" class,
//              we must return FALSE to indicate the DefDlgProc is to
//              process the message further.
//
////////////////////////////////////////////////////////////////////////////
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	/* Save the instance handle in static variable, which will be used in  */
	/* many subsequence calls from this application to Windows.            */

	hInst = hInstance;

	/* Create a main window for this application instance.  */

	ghMainDlg =
	CreateWindow(
				  MAKEINTRESOURCE(0x8002), // Use Windows defined DIALOG class
				  "Drag \& Drop Example",  // Text for window title bar.
				  WS_CAPTION |             // Window style: has a caption
				  WS_VISIBLE |             // Is initially visible
				  WS_SYSMENU |             // Has a system menu
				  WS_THICKFRAME,           // Has a thick frame
				  CW_USEDEFAULT,           // Default horizontal position.
				  CW_USEDEFAULT,           // Default vertical position.
				  CW_USEDEFAULT,           // Default width.
				  CW_USEDEFAULT,           // Default height.
				  NULL,                    // No parent.
              NULL,                    // No menu.
              hInstance,               // This instance owns this window.
              NULL                     // Pointer not needed.
                );                     //End CreateWindow

	/* If window could not be created, return "failure" */

	if (!ghMainDlg)
		return (FALSE);

	/* Set the dialog procedure */

	SetWindowLong (ghMainDlg,
                  DWL_DLGPROC,
                  (LONG) MainDlgProc);    // Function to handle messages

 //
 //Get the dimensions of the dialog box
 //
 RECT rMain ;
 GetClientRect (ghMainDlg, &rMain);

 //
 //Create the list boxes
 //
 ghDragList =                           //Listbox from which items are dragged
 CreateWindow ("ListBox",               //Use the predefined Listbox class
               NULL,                    //No title
               WS_CHILD    |            //Window style: child of main window
               WS_VISIBLE  |            //Initially visible
               WS_DLGFRAME |            //Use the unsizeable dialog frame
               WS_VSCROLL,              //Give it a vertical scroll bar
               rMain.right / 10,        //Position the listbox on the left
               rMain.bottom / 6,        //side of the main window
               (rMain.right / 10) * 3,  //leaving some space between the list
               (rMain.bottom / 6) * 4,  //box edge the the main window edges
               ghMainDlg,               //Parent is the main window
               NULL,                    //No Menu
					hInstance,               //Handle to this application's DS
               NULL );                  //No special bytes

 ghDropList =                           //Listbox to which items are dropped
 CreateWindow ("ListBox",               //Use the predefined Listbox class
               NULL,                    //No title
               WS_CHILD    |            //Window styles: child window
               WS_VISIBLE  |            //Initially visible
               WS_DLGFRAME |            //Use the unsizeable dialog frame
               WS_VSCROLL,              //Give it a vertical scroll bar
               (rMain.right / 10) * 6,  //Place the listbox in the right
               rMain.bottom / 6,        //half of the main window leaving
               (rMain.right / 10) * 3,  //some space between the listbox
               (rMain.bottom / 6) * 4,  //edge and the main window edges
               ghMainDlg,               //Parent is the main window
               NULL,                    //No menu
               hInstance,               //Handle to this application's DS
               NULL );                  //No extra bytes

 //
 //Assign the Window procedure of the first list box.
 //
 DefListBoxProc =                            //Original window procedure
 (LRESULT (FAR PASCAL *)(HWND, UINT, WPARAM, LPARAM))  //Type cast
 SetWindowLong (ghDragList,
                GWL_WNDPROC,
                (LONG) DragListProc);        //New window procedure

 //
 //Mouse is initially not captured
 //
 CaptureOn = FALSE;

 //
 //Add some items to the list
 //
 SendMessage (ghDragList, LB_INSERTSTRING, -1, (LONG) "Sue");
 SendMessage (ghDragList, LB_INSERTSTRING, -1, (LONG) "Janet");
 SendMessage (ghDragList, LB_INSERTSTRING, -1, (LONG) "Jennifer");
 SendMessage (ghDragList, LB_INSERTSTRING, -1, (LONG) "Barbara");
 SendMessage (ghDragList, LB_INSERTSTRING, -1, (LONG) "Cookie");
 SendMessage (ghDragList, LB_INSERTSTRING, -1, (LONG) "Vanida");

	return (TRUE);    // Returns the value from PostQuitMessage
} //End InitInstance


////////////////////////////////////////////////////////////////////////////
//
// FUNCTION:    WinMain ( )
//
// PURPOSE:     Window procedure for the main window of the application
//
// PARAMETERS:  Standard WinMain callback procedure variables,
//              Name          Type        Description
//              ----          ----        -----------
//              hInstance     HINSTANCE  handle to this app's DS
//              hPrevInstance HINSTANCE  handle to any previous of this app
//              lpCmdLine     LPSTR      pointer to the command line
//              nCmdShow      int        constant of how to show window
//                                       eg. Min, Max, Normal
//
// RETURNS:     FALSE. Not used.
//
// REMARKS:     This is the main function for our application.  It executes
//              the main processing loop for messages of the application.
//              It also calls InitInstance which in turn creates the
//              windows displayed in this application.
//
////////////////////////////////////////////////////////////////////////////
int FAR PASCAL WinMain (HINSTANCE hInstance,
                        HINSTANCE hPrevInstance,
			                     LPSTR lpCmdLine,
                        int nCmdShow)
{
	MSG msg;    // message

	/* Perform initializations that apply to a specific instance */

	if (!InitInstance(hInstance, nCmdShow))
		return (FALSE);

	/* Acquire and dispatch messages until a WM_QUIT message is received. */

	while (GetMessage(&msg, // message structure
                   NULL, // handle of window receiving the message
                   NULL, // lowest message to examine
                   NULL))// highest message to examine
	{
		TranslateMessage(&msg); // Translates virtual key codes
		DispatchMessage(&msg);  // Dispatches message to window
	}
	return (msg.wParam); // Returns the value from PostQuitMessage
}  //End WinMain


